package org.vraptor.component;

import java.lang.reflect.Method;
import java.lang.reflect.Modifier;
import java.util.HashMap;
import java.util.Map;

import org.apache.log4j.Logger;
import org.vraptor.annotations.Logic;
import org.vraptor.introspector.ReadParameter;
import org.vraptor.reflection.ReflectionUtil;
import org.vraptor.validator.ValidationErrors;

/**
 * Factory for logic methods.
 * 
 * @author Guilherme Silveira
 */
public class LogicMethodFactory {

	private static final Logger LOG = Logger.getLogger(ComponentTypeFactory.class);

	Map<String, DefaultLogicMethod> loadLogics(Class<?> type) throws InvalidComponentException {
		Map<String, DefaultLogicMethod> actions = new HashMap<String, DefaultLogicMethod>();
		for (Method m : type.getMethods()) {
			if (isLogic(m)) {
				continue;
			}
			Logic annotation = m.getAnnotation(Logic.class);
			generateLogics(actions, m, annotation, type);
		}
		return actions;
	}

	private boolean isLogic(Method m) {
		return !Modifier.isPublic(m.getModifiers()) || Modifier.isStatic(m.getModifiers())
				|| m.getDeclaringClass().equals(Object.class)
				|| m.getName().startsWith(ComponentTypeFactory.VALIDATE_METHOD_INITIALS) || ReflectionUtil.isGetter(m);
	}

	private void generateLogics(Map<String, DefaultLogicMethod> actions, Method method, Logic annotation,
			Class<?> componentClass) throws InvalidComponentException {
		if (annotation == null) {
			register(actions, create(method.getName(), componentClass, method));
		} else {
			for (String name : annotation.value()) {
				register(actions, create(name, componentClass, method));
			}
		}
	}

	private void register(Map<String, DefaultLogicMethod> actions, DefaultLogicMethod action) {
		if (LOG.isDebugEnabled()) {
			LOG.debug("Registering logic " + action.getName());
		}
		actions.put(action.getName(), action);
	}

	public DefaultLogicMethod create(String name, Class<?> type, Method method) {
		return new DefaultLogicMethod(name, method, ReflectionUtil.getPrefixedMethod(type, "validate", name,
				ValidationErrors.class));
	}

}
