package org.vraptor.component;

import java.lang.annotation.Annotation;
import java.lang.reflect.Type;

import org.vraptor.annotations.Parameter;
import org.vraptor.converter.Converter;
import org.vraptor.introspector.ReadParameter;
import org.vraptor.reflection.ReflectionUtil;
import org.vraptor.reflection.SettingException;
import org.vraptor.reflection.StringUtil;

public class MethodParameter implements ReadParameter {

	private final Class<?> paramType;

	private final String key;

	private final int position;

	private final Type generic;

	public MethodParameter(Class<?> param, Type generic, int position, Parameter an) {
		this.paramType = param;
		this.position = position;
		this.generic = generic;
		if (an == null) {
			String name = param.getSimpleName();
			this.key = StringUtil.classNameToInstanceName(name);
		} else {
			this.key = an.key();
		}
	}

	public String getKey() {
		return key;
	}

	public Class<? extends Converter> getOverridenConverter() {
		return null;
	}

	public Object guaranteeExistence(Object component, Object methodParams[]) throws SettingException {
		return methodParams[position];
	}

	public boolean mightCreate() {
		return true;
	}

	public Class<?> getType() {
		return paramType;
	}

	public void set(Object component, Object[] methodParams, Object value) throws SettingException {
		methodParams[position] = value;
	}

	public Type getGenericType() {
		return generic;
	}

	public Object newInstance() throws ComponentInstantiationException {
		return ReflectionUtil.instantiate(this.paramType);
	}

}
