package org.vraptor.config.xml;

import org.vraptor.config.FactoryConfig;

import com.thoughtworks.xstream.converters.Converter;
import com.thoughtworks.xstream.converters.MarshallingContext;
import com.thoughtworks.xstream.converters.UnmarshallingContext;
import com.thoughtworks.xstream.io.HierarchicalStreamReader;
import com.thoughtworks.xstream.io.HierarchicalStreamWriter;

/**
 * A config factory component xml converter.
 * 
 * @author Guilherme Silveira
 */
public class FactoryConfigConverter implements Converter {

	/**
	 * Can it convert this class type?
	 * 
	 * @see com.thoughtworks.xstream.converters.Converter#canConvert(java.lang.Class)
	 */
	public boolean canConvert(Class clazz) {
		return clazz.equals(FactoryConfig.class);
	}

	public void marshal(Object value, HierarchicalStreamWriter writer,
			MarshallingContext context) {
		FactoryConfig object = (FactoryConfig) value;
		writer.setValue(object.getFactoryClass());
	}

	public Object unmarshal(HierarchicalStreamReader reader,
			UnmarshallingContext context) {
		String value = reader.getValue();
		if (value == null) {
			throw new NullPointerException();
		}
		return new FactoryConfig(value);
	}

}
