package org.vraptor.interceptor;

import org.vraptor.Interceptor;
import org.vraptor.LogicException;
import org.vraptor.LogicFlow;
import org.vraptor.LogicRequest;
import org.vraptor.component.ComponentInstantiationException;
import org.vraptor.component.ComponentType;
import org.vraptor.introspector.Introspector;
import org.vraptor.reflection.SettingException;
import org.vraptor.scope.LogicContext;
import org.vraptor.view.ViewException;

/**
 * Interceptor responsible for injection of dependencies on In annotations.
 * 
 * @author Guilherme Silveira
 */
public class InjectionInterceptor implements Interceptor {

	private Introspector introspector;

	private ComponentType type;

	public InjectionInterceptor(LogicContext context, Introspector introspector, ComponentType type) {
		this.introspector = introspector;
		this.type = type;
	}

	public void intercept(LogicFlow flow) throws LogicException, ViewException {

		LogicRequest request = flow.getLogicRequest();
		LogicContext context = request.getLogicContext();

		try {

			Object component = context.getComponent();

			introspector.inject(type.getInAnnotations(), component, request);

		} catch (ComponentInstantiationException e) {
			throw new LogicException(e);
		} catch (SettingException e) {
			throw new LogicException(e);
		}

		flow.execute();

	}

}
