package org.vraptor.plugin.hibernate;

import java.util.ResourceBundle;

import org.apache.log4j.Logger;
import org.hibernate.validator.Valid;
import org.vraptor.LogicRequest;
import org.vraptor.component.LogicMethod;
import org.vraptor.plugin.LogicMethodWrapper;
import org.vraptor.reflection.GettingException;
import org.vraptor.reflection.ReflectionUtil;
import org.vraptor.validator.UnstableValidationException;
import org.vraptor.validator.ValidationErrors;

/**
 * Wrapper for a method which validates itself using hibernate validator
 * annotations.
 * 
 * @author Guilherme Silveira
 * 
 */
class HibernateLogicMethod extends LogicMethodWrapper {

	private static final Logger LOG = Logger.getLogger(HibernateLogicMethod.class);

	private final Validate validate;

	private final ValidatorLocator locator;

	public HibernateLogicMethod(Class containingType, LogicMethod method, ValidatorLocator locator) {
		super(method);
		this.validate = method.getMetadata().getAnnotation(Validate.class);
		this.locator = locator;
	}

	@Override
	public ValidationErrors validate(Object component, LogicRequest context, ResourceBundle bundle)
			throws UnstableValidationException {
		ValidationErrors errors = super.validate(component, context, bundle);
		if (LOG.isDebugEnabled()) {
			LOG.debug("Loading hibernate validation for type " + component.getClass().getName());
		}
		if (bundle != null) {
			bundle = ResourceBundle.getBundle("org.hibernate.validator.resources.DefaultValidatorMessages", bundle
					.getLocale());
		}
		for (String field : validate.fields()) {
			try {
				validateField(component, context, bundle, errors, field);
			} catch (GettingException e) {
				throw new UnstableValidationException("Unable to validate objects", e);
			}
		}
		return errors;
	}

	private void validateField(Object component, LogicRequest context, ResourceBundle bundle, ValidationErrors errors,
			String field) throws GettingException {

		String[] values = field.split("\\.");
		Object current = component;

		for (int i = 0; i < values.length - 1; i++) {
			current = ReflectionUtil.get(current, ReflectionUtil.getField(component.getClass(), values[i]));
		}

		Class<? extends Object> type = current.getClass();
		String fieldName = values[values.length - 1];

		FieldValidation validation;

		if (ReflectionUtil.getField(type, fieldName).isAnnotationPresent(Valid.class)) {
			validation = new RecursiveFieldValidation(type, fieldName, locator);
		} else {
			validation = new EntireFieldValidation(type, fieldName, locator);
		}

		validation.validate(current, context, errors, bundle);

	}

}
