package org.vraptor.webapp;

import java.lang.reflect.Modifier;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import org.apache.log4j.Logger;
import org.vraptor.LogicRequest;
import org.vraptor.component.ComponentManager;
import org.vraptor.component.ComponentNotFoundException;
import org.vraptor.component.ComponentType;
import org.vraptor.component.ComponentTypeFactory;
import org.vraptor.component.InvalidComponentException;
import org.vraptor.component.LogicMethod;

/**
 * A simple implementation of a component manager.
 * 
 * @author Guilherme Silveira
 */
public class DefaultComponentManager implements ComponentManager {

	private static final Logger LOG = Logger.getLogger(DefaultComponentManager.class);

	private Map<String, ComponentType> actions = new HashMap<String, ComponentType>();

	private ComponentTypeFactory componentClassFactory = new ComponentTypeFactory();

	public boolean register(String typeClazz) throws InvalidComponentException {
		try {
			Class<?> type = Class.forName(typeClazz);
			if (Modifier.isPublic(type.getModifiers())) {
				ComponentType component = this.componentClassFactory.getComponentType(type);
				register(component);
				return true;
			} else {
				LOG.info("Ignoring non public class " + typeClazz);
				return false;
			}
		} catch (ClassNotFoundException e) {
			throw new InvalidComponentException("Unable to find type " + typeClazz, e);
		}
	}

	public ComponentType getComponent(LogicRequest request) throws ComponentNotFoundException {
		String completeName = request.getComponentName() + "." + request.getLogicName();
		ComponentType component = this.actions.get(completeName);
		if (component == null) {
			throw new ComponentNotFoundException(String.format("Component %s not found", completeName));
		}
		return component;
	}

	public Set<ComponentType> getComponents() {
		return new HashSet<ComponentType>(actions.values());
	}

	public void register(ComponentType type) {
		LOG.debug("Registering component " + type.getName());
		for (LogicMethod action : type.getLogics()) {
			this.actions.put(String.format("%s.%s", type.getName(), action.getName()), type);
		}
		LOG.debug("Component " + type.getName() + " ok");
	}

}
